// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancyarray.h 825 2010-10-08 09:52:06Z fancylibs $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** An allocated array.
**
*****************************************************************************/

#ifndef FANCYBASE_ARRAY_H
#define FANCYBASE_ARRAY_H

#include <fancyobject.h>

namespace Fancy {

  // if there is no need for a full blown ArrayObject<uint8_t>::ptr_t,
  // Array<uint8_t> can be used leak- and exception-safe byte[] replacement

  template<class T> class Array
  {
    T *t;

  public:

    Array(size_t sz) : t(new T[sz]) {}

    ~Array() { delete[] t; }

    const T &operator[] (size_t i) const { return t[i]; }
    T &operator[] (size_t i) { return t[i]; }

    const T &elem(size_t i) const { return t[i]; }
    T &elem (size_t i) { return t[i]; }

    const T * elem_p(size_t i) const { return t+i; }
    T * elem_p(size_t i) { return t+i; }
  };

  template<class T> class ArrayObject : public Array<T>, virtual public Object
  {
      size_t sz;

  public:
      typedef T *iterator;
      typedef const T *const_iterator;
      typedef Fancy::ObjPtr<ArrayObject<T> > ptr_t;

      static ptr_t newInstance(size_t _sz) { return new ArrayObject(_sz); }

      size_t getSize() const { return sz; }

      iterator begin() { return this->elem_p(0); }
      iterator end() { return this->elem_p(sz); }

      // for primitive types like uint8_t this can be use to get a const char* to the begin of the backing memory block
      const_iterator begin() const { return this->elem_p(0); }
      const_iterator end() const { return this->elem_p(sz); }

      bool lessThan(const Object &o) const {

          const ArrayObject *ao = dynamic_cast<const ArrayObject *>(&o);
          if (!ao) return this->uncomparableLessThan(o);

          if (this->getSize() < ao->getSize()) return true;
          if (this->getSize() > ao->getSize()) return false;

          for (size_t i=0; i< this->getSize();++i) {
              if (this->elem(i) < ao->elem(i)) return true;
              if (ao->elem(i) < this->elem(i)) return false;
          }
          return false;
      }

      bool equals(const Object &o) const {

          const ArrayObject *ao = dynamic_cast<const ArrayObject *>(&o);
          if (!ao) return false;

          if (this->getSize() != ao->getSize()) return false;

          for (size_t i=0; i< this->getSize();++i) {
              if (this->elem(i) != ao->elem(i)) return false;
          }
          return true;
      }

  protected:
      ArrayObject(size_t _sz) : Array<T>(_sz),sz(_sz)
      {}

      ~ArrayObject() {}
  };

}

#endif
