/*
 * $Id: fancybase64bufferchannel.h 909 2011-05-27 21:20:49Z fancylibs $
 *
 * Author: Christoph Lechleitner/IBCL <lech@ibcl.at>
 *
 * Header of Base64BufferChannel
 * A channel reading/writing from/to in-RAM buffers
 *
 */

#ifndef FANCY_BASE64BUFFERCHANNEL_H_
#define FANCY_BASE64BUFFERCHANNEL_H_

#include <fancychannel.h>
#include <fancyobject.h>

namespace Fancy {

template<class T> class ArrayObject;

/*!
 * Base class for Channels that operate on an in-memory buffer and decodes or encodes base64 when writing on it.
 * Also allows direct read acces to the decoded/encoded for easier work on short data blocks like for HTTP Basic Authentication.
 */
class FANCYBASE_API Base64BufferChannel : public virtual Channel {

public:
    typedef ObjPtr<Base64BufferChannel> ptr_t;

    /** returns a Base64BufferChannel instance decoding data on write().
     */
    static ptr_t newDecodingInstance(int bufsize);

    /** returns a Base64BufferChannel instance encoding data on write().
     */
    static ptr_t newEncodingInstance(int bufsize);

    /** copies count bytes of the buffer to data which must have count bytes reserved */
    virtual void getData(void *data, int count) const = 0;

    /** returns a pointer to the data buffer. very dangerous, but useful for i.e. cheap logging. */
    virtual uint8_t* getDataPtr() const = 0;

    /** returns the current read position */
    virtual int getReadPos() const = 0;

    /** returns the current write position */
    virtual int getWritePos() const = 0;

};


}

#endif /* FANCY_BASE64BUFFERCHANNEL_H_ */
