/*
 * fancychannel.h
 *
 *  Created on: 07.05.2009
 *      Author: wglas
 */

#ifndef FANCYCHANNEL_H_
#define FANCYCHANNEL_H_

#include <fancyobject.h>

namespace Fancy {

class Map;

enum ShutdownOption {

    ShutdownRead  = 0,
    ShutdownWrite = 1,
    ShutdownBoth  = 2
};

/*!
 * A read and writable I/O channel like a stream or a file.
 */
class FANCYBASE_API Channel: public virtual Fancy::Object {
public:
    typedef Fancy::ObjPtr<Channel> ptr_t;

    /**
     * Wrap the given channel with a channel, that issues buffered reads and writes
     * on the underlying channel.
     */
    static ptr_t newBufferedInstance(const Channel::ptr_t channel, int bufsize);

    /*!
     * Read a chunk of data from the channel.
     */
    virtual int read (void *buf, int count) = 0;

    /*!
     * Write a chunk of data to the channel.
     */
    virtual int write (const void *buf, int count) = 0;

    virtual void flush() = 0;

    virtual void close() = 0;

    virtual bool isOpen() const = 0;

    /**
     * Shutdown the communication on the channel before actually closing it.
     * In multiprocess enivronments, it may be feasible to not
     * call shutdown from a file descriptor inherited by a child process,
     * because this way the parent process may be interrupted, too.
     * On the other hand, in multi-threaded environments, calling shutdown
     * is a good opportunity to cancel any blocking operation in a concurrent
     * thread.
     *
     * @param o Whether to shut down read, write or all communication.
     */
    virtual void shutdown(ShutdownOption o) = 0;

    /**
     * Append authentication information to the given metdata map.
     * This is mainly used to propagate the X509 certificate of a peer
     * authenticated socket to the application layer.
     *
     * @param metadata A reference to a map, which will be modified.
     *                 If there is metadata to store and metadat is null,
     *                 the implementation should set up a new map containing
     *                 the authentication information and store the new reference
     *                 here.
     */
    virtual void putAuthenticationInfo(Fancy::ObjPtr<Fancy::Map> &metadata) const;

protected:
    Channel() {}
    virtual ~Channel();
};

}

#endif /* FANCYCHANNEL_H_ */
