/*
 * fancycollection.h
 *
 *  Created on: 20.05.2009
 *      Author: wglas
 */

#ifndef FANCYCOLLECTION_H_
#define FANCYCOLLECTION_H_

#include <fancyint.h>
#include <fancyobject.h>

namespace Fancy {

class Iterator;

class FANCYBASE_API Collection: public virtual Fancy::Object {

public:
    typedef ObjPtr<Collection> ptr_t;

    Collection() {}

    /**
     * Return the serial ID of the declared component type of this collection.
     */
    virtual const char *getComponentSerialID() const = 0;
    /**
     * Return the serialVersionUID of the declared component type of this collection or
     * @c (int64_t)-1, if the component type is abstract or a non-serializable
     * base class.
     */
    virtual int64_t getComponentSerialVersionUID() const = 0;

    /**
     * @return The number of elements in this collection.
     */
    virtual size_t getSize() const = 0;

    /**
     * @return An iterator, which points to the first element of the collection.
     */
    virtual ObjPtr<Iterator> getBeginIterator() = 0;

    /**
     * @return An iterator, which points past the last element of the collection.
     */
    virtual ObjPtr<Iterator> getEndIterator() = 0;

    /**
     * Remove all elements.
     */
    virtual void clear() = 0;

protected:
    ~Collection();
};

}

#endif /* FANCYCOLLECTION_H_ */
