/*
 * fancydatastream.h
 *
 *  Created on: 11.05.2009
 *      Author: wglas
 */

#ifndef FANCYDATASTREAM_H_
#define FANCYDATASTREAM_H_

#include <fancychannel.h>
#include <fancyint.h>

namespace Fancy {

class Map;

/**
 * An interface for all reading/writing primitives from/to
 * streams.
 */
class FANCYBASE_API DataStream : public virtual Fancy::Object {
public:
    typedef ObjPtr<DataStream> ptr_t;

    virtual bool    readBoolean() = 0;
    virtual int8_t  readByte() = 0;
    virtual int16_t readShort() = 0;
    virtual int32_t readInt() = 0;
    virtual int64_t readLong() = 0;
    virtual wchar_t readChar() = 0;
    virtual float   readFloat() = 0;
    virtual double  readDouble() = 0;
    virtual std::string readUTF() = 0;
    virtual std::string readUTFLong() = 0;

    virtual void writeBoolean(bool b) = 0;
    virtual void writeByte   (int8_t i) = 0;
    virtual void writeShort  (int16_t i) = 0;
    virtual void writeInt    (int32_t i) = 0;
    virtual void writeLong   (int64_t i) = 0;
    virtual void writeChar   (wchar_t c) = 0;
    virtual void writeFloat  (float f) = 0;
    virtual void writeDouble (double d) = 0;
    virtual void writeUTF    (const char *s, int l=-1) = 0;
    virtual void writeUTFLong(const char *s, int l=-1) = 0;

    virtual void flush() = 0;

    virtual void shutdown(ShutdownOption o) = 0;

    virtual void close() = 0;

    /**
     * Append authentication information to the given metdata map.
     * This is mainly used to propagate the X509 certificate of a peer
     * authenticated socket to the application layer.
     *
     * @param metadata A reference to a map, which will be modified.
     *                 If there is metadata to store and metadata is null,
     *                 the implementation should set up a new map containing
     *                 the authentication information and store the new reference
     *                 here.
     */
    virtual void putAuthenticationInfo(Fancy::ObjPtr<Fancy::Map> &metadata) const;

protected:

    DataStream() {}
    virtual ~DataStream();
};

}

#endif /* FANCYDATASTREAM_H_ */
