/*
 * fancypoolableconnection.h
 *
 *  Created on: 25.11.2009
 *      Author: wglas
 */

#ifndef FANCYPOOLABLECONNECTION_H_
#define FANCYPOOLABLECONNECTION_H_

#include <fancyobject.h>

namespace Fancy {

/**
 * An implementation of this class represents a poolable connection to a
 * remote resource like a remoting client or a database connection.
 */
class FANCYBASE_API PoolableConnection: public virtual Fancy::Object {
public:

    typedef ObjPtr<PoolableConnection> ptr_t;

    /**
     * @return Whether this instance is currently connected.
     */
    virtual bool isConnected() const = 0;

    /**
     * Disconnect this remote resources. Calling disconnect is
     * needed even, if this instance received a transport error.
     */
    virtual void disconnect() = 0;

    /**
     * connect this remote resource.
     */
    virtual void connect() = 0;

    /**
     * @return Whether this instance has received an error on the transport layer.
     *         If @c true, it is most likely advisable to reconnect this instance.
     */
    virtual bool hasTransportError() const = 0;

protected:
    PoolableConnection() {}
    virtual ~PoolableConnection();
};

class FANCYBASE_API PoolableConnectionFactory: public virtual Fancy::Object {
public:

    typedef ObjPtr<PoolableConnectionFactory> ptr_t;

    /**
     * @return A new poolable connection instance.
     */
    virtual PoolableConnection::ptr_t newConnection() = 0;
protected:
    PoolableConnectionFactory() {}
    virtual ~PoolableConnectionFactory();
};

}

#endif /* FANCYPOOLABLECONNECTION_H_ */
